"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

from prompt_toolkit.application import Application
from prompt_toolkit.layout import Layout
from prompt_toolkit.output.color_depth import ColorDepth
from prompt_toolkit.styles.pygments import style_from_pygments_cls
from .style import SolarizedStyle
from .interface import input_field, container, command_processor
from .key_bindings import key_bindings


style = style_from_pygments_cls(SolarizedStyle)

app = Application(layout=Layout(container,
                  focused_element=input_field),
                  key_bindings=key_bindings,
                  style=style,
                  include_default_pygments_style=False,
                  color_depth=ColorDepth.DEPTH_24_BIT,
                  full_screen=True)



# flake8: noqa
"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

import click
from platform import python_version
from prompt_toolkit.application import Application
from prompt_toolkit.document import Document
from prompt_toolkit.filters import has_focus
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.layout.containers import HSplit, Window
from prompt_toolkit.layout.layout import Layout
from prompt_toolkit.styles import Style
from prompt_toolkit.widgets import TextArea, SearchToolbar
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.lexers import PygmentsLexer
from pygments.lexers import JsonLexer
from .command_processor import CommandProcessor, Command


# Utilities
# ---------
def init_text():
    """Returns the introductory text applied to output when the app is launched."""
    return (u'[wsh "{}"] (Python "{}")\n'.format(__version__, python_version()) +
            u'\nType "help" for more info, or "exit" to quit.\n' +
            u'-' * click.get_terminal_size()[0])


# Command Processor
command_processor = CommandProcessor()

# Output Field
# ------------
output_field = TextArea(text=init_text(),
                        lexer=PygmentsLexer(JsonLexer))


# Input Field
# -----------
def accept_handler(buf):
    """Callback method invoked when <ENTER> is pressed."""
    command_processor.accept_handler(buf, input_field, output_field)


command_completer = WordCompleter([command.value for command in Command],
                                  ignore_case=True)

input_field = TextArea(height=1,
                       prompt=u'>>> ',
                       lexer=PygmentsLexer(JsonLexer),
                       completer=command_completer,
                       style='class:input-field',
                       multiline=False,
                       wrap_lines=False,
                       accept_handler=accept_handler)

# Container
# ---------
container = HSplit([output_field,
                    Window(height=1, char=u'-', style=u'class:line'),
                    input_field])



#!/usr/bin/env python
"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

__version__ = '0.1.0'

from blinker import signal
from enum import Enum, unique
from prompt_toolkit.document import Document
from prompt_toolkit.application.current import get_app


@unique
class Command(Enum):
    EXIT = 'exit'
    NONE = ''
    SEND = 'send'
    CLOSE = 'close'
    HELP = 'help'


class CommandProcessor:

    def __init__(self):
        self.senders = dict()
        self.receivers = dict()

    def process_command(self, command, argument, input_field, output_field):
        if command == Command.NONE:
            input_field.text = u""
        elif command == Command.EXIT:
            get_app().exit()
        elif command == Command.SEND:
            wsh_send = signal('wsh-send')
            wsh_send.send(self, data=argument)
        elif command == Command.CLOSE:
            wsh_close = signal('wsh-close')
            wsh_close.send(self)
        elif command == Command.HELP:
            self.show_help(output_field)

    def accept_handler(self, buf, input_field, output_field):
        """Callback method invoked when <ENTER> is pressed."""
        try:
            tokens = input_field.text.split(' ')

            raw_command, sep, argument = input_field.text.partition(' ')
            if raw_command in self.senders:
                wsh_send = signal('wsh-send')
                wsh_send.send(self, data=self.senders[raw_command](argument))
                return

            command = Command(tokens[0])
            self.process_command(command, argument, input_field, output_field)
        except ValueError:
            output = u'Command not found, type "help" to see available commands.'
            output = output_field.text + '\n' + output
            output_field.buffer.document = Document(text=output,
                                                    cursor_position=len(output))
        finally:
            input_field.text = u""

    def register_sender(self, name, command):
        """Register a new command as a sending function in the shell."""
        self.senders[name] = command

    def register_receiver(self, name, callback):
        """Register a new command as a receiving callback in the shell."""
        self.receivers[name] = callback

    def show_help(self, output_field):
        """Shows a help message that lists available commands."""
        output = """
        `exit`           - Exit the shell.
        `send <data>`    - Sends the given data to the server.
        `close`          - Close the connection to the server but dont exit.
        `help`           - Shows this help message.
        """
        output = output_field.text + '\n' + output
        output_field.buffer.document = Document(text=output,
                                                cursor_position=len(output))



"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

from prompt_toolkit.key_binding import KeyBindings


key_bindings = KeyBindings()


@key_bindings.add(u'c-c')
@key_bindings.add(u'c-q')
def _(event):
    " Pressing Ctrl-Q or Ctrl-C will exit the user interface. "
    event.app.exit()
